//*******************************************************
//
//	aitendo AD-12864-SPI LCD (ST7565)
//
//	2010.11.30	KAWAKAMI Yukio
//
//	New BSD license
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"radio.h"
#include	"LCDfont.h"
#include	"NumFont.h"
#include	"lcd.h"

#include	<util/delay.h>


uchar	CurX, CurY;
#ifdef USE_LCD_BUFF
uchar	LcdBuff[128*64/8];	// LCDバッファ 1024bytes
#endif


//********************************************************************
//	周波数表示用文字列出力(数字と一部記号のみ)
//********************************************************************
void print_large_num(char x, char y, char *str){
	char	data;
	char	w, h;
	PGM_P	p;

	while((data = *str++)){
		if (isdigit(data)){
			p = (PGM_P)&NumData[data - '0'];
		} else if (data == ' ') {
			p = (PGM_P)&NumData[10];
		} else if (data == '.') {
			p = (PGM_P)&NumData[11];
		} else if (data == 'A') {
			p = (PGM_P)&NumData[12];
		} else if (data == 'F') {
			p = (PGM_P)&NumData[13];
		} else if (data == 'M') {
			p = (PGM_P)&NumData[14];
		} else if (data == 'W') {
			p = (PGM_P)&NumData[15];
		} else if (data == 'k') {
			p = (PGM_P)&NumData[16];
		} else if (data == 'H') {
			p = (PGM_P)&NumData[17];	// Hzが一つのフォントに入っている
		} else if (data == 'z') {
			continue;					// のでこちらは無視
		} else {
			continue;
		}
		for (h = 0; h < 16/8; h++){
			LCD_set(x, y/8+h);
			for (w = 0; w < 10; w++){
				LCD_data(pgm_read_byte(p++));
			}
		}
		x += 10;
	}
}


//********************************************************************
//	ASCIIフォント出力(6 x 8ドット)
//********************************************************************
void putch(char ch){

	if (ch == CR){
		CurX = 0;
	} else if (ch == LF){
		CurX = 0;
		if (CurY < LCD_HEIGHT)	CurY += 8;
	} else {
		if ((ch < ' ')||(ch > 0x7F)){
			ch = '.';
		}
		if (CurX > (LCD_WIDTH - 6)){
			CurX = 0;
			if (CurY < LCD_HEIGHT)	CurY += 8;
		}
		if (CurY < (LCD_HEIGHT - 7)){
			PGM_P	p;
			uchar	i;
#ifdef USE_LCD_BUFF
			uchar	*bp;
#endif
			uchar	d;

			if ((CurY & 7) == 0){
				// マスク処理不要

				LCD_set(CurX, CurY/8);
#ifdef USE_LCD_BUFF
				bp = &LcdBuff[(CurY/8)*LCD_WIDTH+CurX];
#endif
				p = (PGM_P)&(AsciiFont[((int)(ch - 0x20)) * 5]);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++);
					LCD_data(d);
#ifdef USE_LCD_BUFF
					*bp++ = d;
#endif
				}
				LCD_data(0);
#ifdef USE_LCD_BUFF
				*bp = 0;
#endif
			} else {
#ifdef USE_LCD_BUFF
				// マスク処理必要
				uchar	mask;
				uchar	shift;

				shift = CurY & 7;

				mask = ~(0xFF << shift);
				bp = &LcdBuff[((int)CurY/8)*LCD_WIDTH+CurX];
				p = (PGM_P)&(AsciiFont[((int)(ch - 0x20)) * 5]);
				LCD_set(CurX, CurY/8);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++) << shift;
					d |= (*bp & mask);
					LCD_data(d);
					*bp++ = d;
				}
				d = (*bp & mask);
				LCD_data(d);
				*bp = d;

				bp += LCD_WIDTH - 5;
				
				mask = 0xFF << shift;
				p -= 5;
				LCD_set(CurX, CurY/8+1);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++) >> (8 - shift);
					d |= (*bp & mask);
					LCD_data(d);
					*bp++ = d;
				}
				d = (*bp & mask);
				LCD_data(d);
				*bp = d;
#endif
			}
			CurX += 6;
		}
	}
}


//*******************************************************
//	カーソル位置設定
//*******************************************************
void locate(uchar x, uchar y){

	CurX = x * 6;
	CurY = y * 8;
}


//*******************************************************
//	文字列表示
//*******************************************************
void print(char *str){
	char	c;

	while((c = *str++)){
		putch(c);
	}
}


//*******************************************************
//	固定文字列表示
//*******************************************************
void print_P(PGM_P str){
	char	c;

	while((c = pgm_read_byte(str++))){
		putch(c);
	}
}


//*********************************************************
//	LCD RAMの書き込み開始位置セット
//	xは1ピクセル単位、yは8ピクセル単位
//*********************************************************
void LCD_set(uchar x, uchar y){

	LCD_command(0x10 + (x >> 4));
	LCD_command(x & 0x0F);
	LCD_command(0xB0 + y);
}


//*********************************************************
//	LCD消去
//*********************************************************
void clear_LCD(void){
	uchar	x, y;

	for (y = 0; y < LCD_HEIGHT/8; y++){
		LCD_set(0, y);
		for (x = 0; x < LCD_WIDTH; x++){
			LCD_data(0);
		}
	}
	CurX = CurY = 0;
}


//*********************************************************
//	LCD初期化
//*********************************************************
void LCD_init(void){

	PORTB &= ~(1<<LRES);	// LCDリセット
	_delay_ms(1);
	PORTB |= (1<<LRES);		// LCDリセット解除
	_delay_ms(1);

	LCD_command(0xA2);		// LCD Bias		1/9
	LCD_command(0xA0);		// ADC Select	normal
	LCD_command(0xC8);		// Common Output Mode	reverse
	LCD_command(0x24);		// V5 Voltage Regulator Internal Resistor Ratio
	LCD_command(0x81);		// Electronic Volume Mode
	LCD_command(0x1D);		// default contrast
	LCD_command(0x2F);		// Power Controller	all ON
	LCD_command(0x40);		// Display Start Line	0
	LCD_command(0xE0);		// Read/Modify/Write Mode Set

	clear_LCD();

	LCD_command(0xAF);		// Display ON
}


//*********************************************************
//	LCD 電源OFF
//*********************************************************
void LCD_off(void){
	
	PORTB &= ~(1<<LRES);	// LCDリセット
}


//*********************************************************
//	LCDコントラスト設定
//*********************************************************
void LCD_contrast(uchar contrast){

	if(contrast < 1) contrast = 1;
	if(contrast > 0x3F) contrast = 0x3F;

	LCD_command(0x81);			// Set electronic volume mode
	LCD_command(contrast);
}



//********************************************************************
//	LCD コマンド出力
//********************************************************************
void LCD_command(uchar com){

	PORTB &= ~((1<<CS1)|(1<<A0));
	SPI_out(com);
	PORTB |= (1<<CS1);
}


//********************************************************************
//	LCD データ出力
//********************************************************************
void LCD_data(uchar data){

	PORTB &= ~(1<<CS1);
	PORTB |= (1<<A0);
	SPI_out(data);
	PORTB |= (1<<CS1);
}


