#ifndef	__RADIO_H__
#define	__RADIO_H__
//*******************************************************
//
//	ワイドバンド AM/FMラジオ / サイレントシステム SR-1
//
//	ATmega328p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.4
//				AVR-LIBC	1.6.7
//				binutils	2.20
//
//	2010.12.3	KAWAKAMI Yukio
//
//	New BSD license
//*******************************************************

#ifndef		F_CPU
#define		F_CPU	8000000UL
#endif
#define		CPU_CLOCK	F_CPU

#define		VERSION	"1.0.1"

#define		CR	13
#define		LF	10
#define		TAB	9
#define		ON	1
#define		OFF	0
#define		TRUE	1
#define		FALSE	0

#define		FAT12		1
#define		FAT16		2

#define		BEEP_HIGH	0
#define		BEEP_LOW	1

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;

#define		PLOCK	7	// PORTD
#define		AMP		6
#define		ENC2	5
#define		ENC1	4
#define		SW6		3
#define		SW5		2
#define		TXD		1
#define		RXD		0

#define		LED		5	// PORTC
#define		ANT		4
#define		SW4		3
#define		SW3		2
#define		SW2		1
#define		SW1		0

#define		NC1		7	// PORTB
#define		NC2		6
#define		SCK		5
#define		MISO	4
#define		MOSI	3
#define		CS1		2
#define		A0		1
#define		LRES	0

#define		VOL			7	// ADC7
#define		BATT_VOLT	6	// ADC6

#define		R5_VALUE	22
#define		R6_VALUE	10

#define		BAND_MW		0
#define		BAND_AM		1
#define		BAND_FM		2

#define		STEP_LOW	0
#define		STEP_MIDDLE	1
#define		STEP_HIGH	2

#define		EEPROM_BAND		(uint8_t *)1
#define		EEPROM_STEP_AM	(uint8_t *)2
#define		EEPROM_STEP_FM	(uint8_t *)3
#define		EEPROM_FREQ_MW	(uint16_t *)4
#define		EEPROM_FREQ_AM	(uint16_t *)6
#define		EEPROM_FREQ_FM	(uint16_t *)8

#define		SW_NONE		0
#define		SW_POWER	1
#define		SW_MODE		2
#define		SW_SW3		3	// 仮
#define		SW_SW4		4	// 仮
#define		SW_SW5		5	// 仮
#define		SW_STEP		6

int read_batt(void);
uchar read_vol(void);
int *set_band(char band, int *step, int *min, int *max);

uchar check_sw(void);
void wait_sw_off(void);
uchar read_encorder(void);
void print_response(void);
void wait_prompt(void);
void power_off(void);
void wait_timer(uchar t);
void SPI_out(uchar data);


#endif	// __RADIO_H__
